function [S,A]=ica_bell2(X,LearnRateMethode,LearnRate,StopDiffEntropi,Display);
% ica_bell2.m           : ICA decomp using infomax by Bell with natural gradient and linesearch
%
%                         [S,A]=ica_bell2(X,LearnRateMethode,LearnRate,StopDiffEntropi,Display)
%            
%                         X  : Input matrix (Rows are signals and Cols are samples)                     
%                         LearnRateMethode :  1: Linesearch from top from LearnRate (ex. 1e-2)
%                                             2: Same stepsize = Learnrate (ex. 1e-4)
%                         LearnRate        : See LearnRateMethode.
%                         StopDiffEntropi  : Stop criteria with difference in the entropi between iteration (ex. 1e-14)
%                         Display          : Display entropi and graphs
%                                             0: No output
%                                             1: Output to commandline
%                                             2: As 1 and entropi (costfunction) graph
%
%                         S                : ICA components: X=A*S
%                         A                : Mixing matrix
%
% author Thomas Kolenda 1998,2000 IMM DTU


% ** Parameters ***************************************************************

BiasLearnRate=0;	% Bias is not used 
LRDivMax=100;		% Stop crireria for number of divisions of the maximum learnrate
iterations=10000; % Stop criteria when iterating more than given times

% Store origenal input
Xin=X;

% ** Initialize ***************************************************************
disp('[Initialize]')
[M,N]=size(X);
disp(sprintf('[Number of signals %d and Number of sampels %d]',M,N))
disp(sprintf('[Rank of input %d]',rank(X)))

Wo=zeros(M,1);
W=eye(M);

Y=zeros(M,N);
olddW=ones(M);
dolddW=ones(M);
I=eye(M);

LRDiv=0;
LR=0;
BLR=BiasLearnRate;

oldE=-inf;
preE=0;
diffE=inf;

% ** Start optimize ***********************************************************
disp('[Running ica decomp...]');

% Estimate output
[Y,U]=map(X,W,Wo);
entropi=relentropi(Y,W);

j=0;
while (j<=iterations) & (diffE>StopDiffEntropi) & (LRDiv<LRDivMax),
  oldE=entropi;
  j=j+1;
  
  % Calc Natural gradient direction
  Yh = 1-2*Y;
  dW = ((N*I+Yh*X'*W')*W); 
  dWo = (sum(Yh')');
  
  % Linesearch - find stepsize 
  switch LearnRateMethode,
    case 1, [LR,BLR,LRDiv]=LineSearchTop(W,Wo,dW,dWo,X,Y,LearnRate,BiasLearnRate,LRDivMax);
    case 2, LR=LearnRate;
  end;
  
  % Make Step
  W=W + LR*dW;
  Wo=Wo + BLR*dWo;
  [Y,U]=map(X,W,Wo);
  
  % Paint
  %[entropi]=showe(X,W,Wo);
  entropi=relentropi(Y,W);
  diffE=abs(entropi-preE);
  if Display==1 | Display==2,
    disp(sprintf('#:   %d    Rel Entropi: %d    Diff Entropi: %d    LRDiv: %d  \n',j,entropi,diffE,LRDiv));
  end
  preE=entropi;

  E(j)=entropi;
  if Display==2,
    figure(1);
    plot(1:length(E),E,'b')
  end
end

A=inv(W);
S=U;

disp('[Ica done]');


% *****************************************************************************
function E=relentropi(Y,W)
% relentropi.m      : Calc the relative entropi
%
%				           relentropi(Y,W)
%
%                      Y: Output of squashing function 
%                      W: Seperationmatrix

psi=1e-8;
E=mean(log(psi+abs(det(W)*prod(Y.*(1-Y)))));



function [Y,U]=map(X,W,Wo)
% Map.m             : Calc the output from the ica mapping
%
%                      [Y,U]=Map(X,W,Wo)
%                    
%                      X  : Mixed signals
%                      W  : Weights
%                      Wo : Bias weights
%
%                      Y  : Output of squashing function
%                      U  : Output of seperation
U=W*X+Wo*ones(1,size(X,2));
Y=1./(1+exp(-U));




function [LR,BLR,LRDiv]=LineSearchTop(W,Wo,dW,dWo,X,Y,LearnRate,BiasLearnRate,LRMax)
% LineSearchTop.m   : Linesearch from top, starting with a large value
%
%                       [LR,BLR,LRDiv]=LineSearchTop(W,Wo,dW,dWo,X,Y,LearnRate,BiasLearnRate,LRMax)
%
%                       W                  : Separation matrix
%                       Wo                 : Separation matrix bias
%                       dW                 : New step for separation matrix
%                       dWo                : New step for separation matrix bias
%                       X                  : Mixed signals
%                       Y                  : Output of squashing function
%                       LearnRate          : Start learnrate
%                       BiasLearnRate      : Start learnrate for bias
%                       LRMax              : maximum number of divisions
%
%                       LR                 : Result learnrate
%                       BLR                : Result learnrate for bias
%                       LRDiv              : Number of divisions og start learnrate

oldW=W; oldWo=Wo;
LR=LearnRate; BLR=BiasLearnRate; LRDiv=0;

entropi=relentropi(Y,W);
oldE=entropi;
while (entropi<=oldE) & ((LRDiv-1)<=LRMax),
   W=oldW + LR*dW;
   [Y,U]=map(X,W,Wo);
   entropi=relentropi(Y,W);
   LR=LR/2; LRDiv=LRDiv+1;
end
%LR=LR*2; LRDiv=LRDiv-1;



